"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from orb import utils
from typing import Any, Optional



@dataclasses.dataclass
class RequestValidationErrorHeaders:
    pass

class RequestValidationErrorStatus(int, Enum):
    FOUR_HUNDRED = 400

class RequestValidationErrorType(str, Enum):
    HTTPS_DOCS_WITHORB_COM_REFERENCE_ERROR_RESPONSES_NUMBER_400_REQUEST_VALIDATION_ERRORS = 'https://docs.withorb.com/reference/error-responses#400-request-validation-errors'


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class RequestValidationError:
    status: RequestValidationErrorStatus = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('status') }})
    type: RequestValidationErrorType = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('type') }})
    validation_errors: list[Any] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('validation_errors') }})
    detail: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('detail') }})
    headers: Optional[RequestValidationErrorHeaders] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('headers') }})
    title: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('title') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from .five_hundrederror import *
from .four_hundred_and_fourerror import *
from .four_hundred_and_nineerror import *
from .four_hundred_and_oneerror import *
from .four_hundred_and_thirteenerror import *
from .four_hundred_and_twenty_nineerror import *
from .four_hundrederror import *
from .sdkerror import SDKError
__all__ = ["FiveHundredError","FiveHundredErrorHeaders","FourHundredAndFourError","FourHundredAndNineError","FourHundredAndOneError","FourHundredAndOneErrorHeaders","FourHundredAndThirteenError","FourHundredAndTwentyNineError","FourHundredAndTwentyNineErrorHeaders","FourHundredError","SDKError"]



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from orb import utils


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class Debug:
    r"""Optional debug information (only present when debug=true is passed to the endpoint). Contains ingested and duplicate event idempotency keys."""
    duplicate: list[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('duplicate') }})
    ingested: list[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('ingested') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from ..shared import accountingsyncconfiguration as shared_accountingsyncconfiguration
from ..shared import address as shared_address
from ..shared import customertaxid as shared_customertaxid
from ..shared import reportingconfiguration as shared_reportingconfiguration
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from enum import Enum
from orb import utils
from typing import Optional

class CustomerPaymentProvider(str, Enum):
    r"""This is used for creating charges or invoices in an external system via Orb. When not in test mode, the connection must first be configured in the Orb webapp."""
    QUICKBOOKS = 'quickbooks'
    BILL_COM = 'bill.com'
    STRIPE_CHARGE = 'stripe_charge'
    STRIPE_INVOICE = 'stripe_invoice'
    NETSUITE = 'netsuite'


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class Customer:
    r"""A customer is a buyer of your products, and the other party to the billing relationship.

    In Orb, customers are assigned system generated identifiers automatically, but it's often desirable to have these
    match existing identifiers in your system. To avoid having to denormalize Orb ID information, you can pass in an
    `external_customer_id` with your own identifier. See
    [Customer ID Aliases](../guides/events-and-metrics/customer-aliases) for further information about how these
    aliases work in Orb.

    In addition to having an identifier in your system, a customer may exist in a payment provider solution like
    Stripe. Use the `payment_provider_id` and the `payment_provider` enum field to express this mapping.

    A customer also has a timezone (from the standard [IANA timezone database](https://www.iana.org/time-zones)), which
    defaults to your account's timezone. See [Timezone localization](../guides/product-catalog/timezones.md) for
    information on what this timezone parameter influences within Orb.
    """
    additional_emails: list[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('additional_emails') }})
    auto_collection: bool = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('auto_collection') }})
    balance: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('balance') }})
    r"""The customer's current balance in their currency."""
    billing_address: Optional[shared_address.Address] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('billing_address') }})
    created_at: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('created_at'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    currency: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('currency') }})
    email: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('email') }})
    r"""A valid customer email, to be used for notifications. When Orb triggers payment through a payment gateway, this email will be used for any automatically issued receipts."""
    email_delivery: bool = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('email_delivery') }})
    external_customer_id: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('external_customer_id') }})
    r"""An optional user-defined ID for this customer resource, used throughout the system as an alias for this Customer. Use this field to identify a customer by an existing identifier in your system."""
    id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id') }})
    metadata: dict[str, str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata') }})
    name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    r"""The full name of the customer"""
    payment_provider: Optional[CustomerPaymentProvider] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('payment_provider') }})
    r"""This is used for creating charges or invoices in an external system via Orb. When not in test mode, the connection must first be configured in the Orb webapp."""
    payment_provider_id: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('payment_provider_id') }})
    r"""The ID of this customer in an external payments solution, such as Stripe. This is used for creating charges or invoices in the external system via Orb."""
    portal_url: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('portal_url') }})
    shipping_address: Optional[shared_address.Address] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('shipping_address') }})
    tax_id: Optional[shared_customertaxid.CustomerTaxID] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('tax_id') }})
    timezone: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('timezone') }})
    r"""A timezone identifier from the IANA timezone database, such as \\"America/Los_Angeles\\". This \\"defaults to your account's timezone if not set. This cannot be changed after customer creation."""
    accounting_sync_configuration: Optional[shared_accountingsyncconfiguration.AccountingSyncConfiguration] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('accounting_sync_configuration') }})
    reporting_configuration: Optional[shared_reportingconfiguration.ReportingConfiguration] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('reporting_configuration') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from datetime import date
from orb import utils
from typing import Union



@dataclasses.dataclass
class CreateInvoiceLineItemParamsQuantity:
    pass


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class CreateInvoiceLineItemParams:
    amount: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('amount') }})
    r"""The total amount in the invoice's currency to add to the line item."""
    end_date: date = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('end_date'), 'encoder': utils.dateisoformat(False), 'decoder': utils.datefromisoformat }})
    r"""A date string to specify the line item's end date in the customer's timezone."""
    invoice_id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('invoice_id') }})
    r"""The id of the Invoice to add this line item."""
    name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    r"""The item name associated with this line item. If an item with the same name exists in Orb, that item will be associated with the line item."""
    quantity: Union[float, str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('quantity') }})
    r"""The number of units on the line item"""
    start_date: date = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('start_date'), 'encoder': utils.dateisoformat(False), 'decoder': utils.datefromisoformat }})
    r"""A date string to specify the line item's start date in the customer's timezone."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from ..shared import priceintervalfixedfeequantitytransitionparams as shared_priceintervalfixedfeequantitytransitionparams
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from enum import Enum
from orb import utils
from typing import Optional, Union

class EditPriceIntervalParamsEndDateEndDate(str, Enum):
    r"""The updated end date of this price interval. If not specified, the start date will not be updated."""
    START_OF_TERM = 'start_of_term'
    END_OF_TERM = 'end_of_term'



@dataclasses.dataclass
class EditPriceIntervalParamsEndDate:
    pass

class EditPriceIntervalParamsStartDateStartDate(str, Enum):
    r"""The updated start date of this price interval. If not specified, the start date will not be updated."""
    START_OF_TERM = 'start_of_term'
    END_OF_TERM = 'end_of_term'



@dataclasses.dataclass
class EditPriceIntervalParamsStartDate:
    pass


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class EditPriceIntervalParams:
    price_interval_id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('price_interval_id') }})
    r"""The id of the price interval to edit."""
    end_date: Optional[Union[datetime, EditPriceIntervalParamsEndDateEndDate]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('end_date') }})
    r"""The updated end date of this price interval. If not specified, the start date will not be updated."""
    fixed_fee_quantity_transitions: Optional[list[shared_priceintervalfixedfeequantitytransitionparams.PriceIntervalFixedFeeQuantityTransitionParams]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('fixed_fee_quantity_transitions') }})
    r"""A list of fixed fee quantity transitions to use for this price interval. Note that this list will overwrite all existing fixed fee quantity transitions on the price interval."""
    start_date: Optional[Union[datetime, EditPriceIntervalParamsStartDateStartDate]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('start_date') }})
    r"""The updated start date of this price interval. If not specified, the start date will not be updated."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from orb import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class Address:
    city: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('city') }})
    country: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('country') }})
    line1: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('line1') }})
    line2: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('line2') }})
    postal_code: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('postal_code') }})
    state: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('state') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from .sdkconfiguration import SDKConfiguration
from orb import utils
from orb.models import errors, operations, shared
from typing import Optional

class Item:
    r"""The Item resource represents a sellable product or good. Items are associated with all line items, billable metrics,
    and prices and are used for defining external sync behavior for invoices and tax calculation purposes.
    """
    sdk_configuration: SDKConfiguration

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        self.sdk_configuration = sdk_config
        
    
    def fetch(self, item_id: str) -> operations.FetchItemResponse:
        r"""Fetch item
        This endpoint returns an item identified by its item_id.
        """
        request = operations.FetchItemRequest(
            item_id=item_id,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = utils.generate_url(operations.FetchItemRequest, base_url, '/items/{item_id}', request)
        headers = {}
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        
        client = self.sdk_configuration.security_client
        
        http_res = client.request('GET', url, headers=headers)
        content_type = http_res.headers.get('Content-Type')

        res = operations.FetchItemResponse(status_code=http_res.status_code, content_type=content_type, raw_response=http_res)
        
        if http_res.status_code == 200:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, Optional[shared.Item])
                res.item = out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 401:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndOneError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 404:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndFourError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 409:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 413:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndThirteenError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 429:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndTwentyNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 500:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FiveHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)

        return res

    
    def list(self, cursor: Optional[str] = None, limit: Optional[int] = None) -> operations.ListItemsResponse:
        r"""List items
        This endpoint returns a list of all Items, ordered in descending order by creation time.
        """
        request = operations.ListItemsRequest(
            cursor=cursor,
            limit=limit,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/items'
        headers = {}
        query_params = utils.get_query_params(operations.ListItemsRequest, request)
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        
        client = self.sdk_configuration.security_client
        
        http_res = client.request('GET', url, params=query_params, headers=headers)
        content_type = http_res.headers.get('Content-Type')

        res = operations.ListItemsResponse(status_code=http_res.status_code, content_type=content_type, raw_response=http_res)
        
        if http_res.status_code == 200:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, Optional[shared.Items])
                res.items = out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 401:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndOneError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 404:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndFourError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 409:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 413:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndThirteenError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 429:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndTwentyNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 500:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FiveHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)

        return res

    


"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from datetime import date
from enum import Enum
from orb import utils
from typing import Optional, Union



@dataclasses.dataclass
class AddExpirationChangeCreditLedgerEntryRequestParamsAmount:
    pass

class AddExpirationChangeCreditLedgerEntryRequestParamsEntryType(str, Enum):
    EXPIRATION_CHANGE = 'expiration_change'



@dataclasses.dataclass
class AddExpirationChangeCreditLedgerEntryRequestParamsMetadata:
    r"""User-specified key/value pairs for the ledger entry resource."""
    



@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class AddExpirationChangeCreditLedgerEntryRequestParams:
    entry_type: AddExpirationChangeCreditLedgerEntryRequestParamsEntryType = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('entry_type') }})
    expiry_date: Optional[date] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('expiry_date'), 'encoder': utils.dateisoformat(False), 'decoder': utils.datefromisoformat }})
    r"""A future date (specified in YYYY-MM-DD format) that identifies the origination credit block to expire"""
    target_expiry_date: date = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('target_expiry_date'), 'encoder': utils.dateisoformat(False), 'decoder': utils.datefromisoformat }})
    r"""A future date (specified in YYYY-MM-DD format) used for expiration change, denoting when credits transferred (as part of a partial block expiration) should expire."""
    amount: Optional[Union[float, str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('amount') }})
    r"""The number of credits to effect. Note that this is required for increment, decrement, void, or undo operations."""
    block_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('block_id') }})
    r"""The ID of the block affected by an expiration_change, used to differentiate between multiple blocks with the same `expiry_date`."""
    description: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('description') }})
    r"""Optional metadata that can be specified when adding ledger results via the API. For example, this can be used to note an increment refers to trial credits, or for noting corrections as a result of an incident, etc."""
    metadata: Optional[AddExpirationChangeCreditLedgerEntryRequestParamsMetadata] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    r"""User-specified key/value pairs for the ledger entry resource."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

import requests as requests_http
from .availability import Availability
from .coupon import Coupon
from .credit import Credit
from .credit_note import CreditNote
from .customer import Customer
from .event import Event
from .invoice import Invoice
from .item import Item
from .metric import Metric
from .plan import Plan
from .price import Price
from .price_interval import PriceInterval
from .sdkconfiguration import SDKConfiguration
from .subscription import Subscription
from orb import utils
from orb.models import shared

class Orb:
    r"""API Reference: Orb's API is built with the following principles in mind:

    1. **Predictable developer experience**: Where applicable, the Orb API uses industry-standard patterns such as
    cursor-based pagination and standardized error output. To help with debugging in critical API
    actions, the API always strives to provide detailed and actionable error messages. Aliases
    such as external customer IDs aid in fast integration times.
    2. **Reliably real time**: Orb's event-based APIs, such as event ingestion are designed to handle extremely high
    throughput and scale with concurrent load. Orb also provides a real-time event-level credits
    ledger and a highly performant webhooks architecture.
    3. **Flexibility at the forefront**: Features like timezone localization and the ability to amend historical usage
    show the flexible nature of the platform.

    You can download the latest OpenAPI spec [here](pathname:///spec.json).
    """
    availability: Availability
    coupon: Coupon
    r"""A coupon represents a reusable discount configuration, and have an attached redemption code that can be issued to
    your end users. Coupons are most often used in self-serve signup or upgrade flows in your checkout experience or
    billing portal.

    To redeem a coupon, pass the `redemption_code` property in the [create subscription](create-subscription.api.mdx)
    or [schedule plan change](schedule-plan-change.api.mdx) request.
    """
    credit: Credit
    r"""The [Credit Ledger Entry resource](/guides/product-catalog/prepurchase) models prepaid credits within Orb."""
    credit_note: CreditNote
    r"""The [Credit Note](/guides/invoicing/credit-notes) resource represents a credit that has been applied to a
    particular invoice.
    """
    customer: Customer
    r"""A customer is a buyer of your products, and the other party to the billing relationship.

    In Orb, customers are assigned system generated identifiers automatically, but it's often desirable to have these
    match existing identifiers in your system. To avoid having to denormalize Orb ID information, you can pass in an
    `external_customer_id` with your own identifier. See
    [Customer ID Aliases](../guides/events-and-metrics/customer-aliases) for further information about how these
    aliases work in Orb.

    In addition to having an identifier in your system, a customer may exist in a payment provider solution like
    Stripe. Use the `payment_provider_id` and the `payment_provider` enum field to express this mapping.

    A customer also has a timezone (from the standard [IANA timezone database](https://www.iana.org/time-zones)), which
    defaults to your account's timezone. See [Timezone localization](../guides/product-catalog/timezones.md) for
    information on what this timezone parameter influences within Orb.
    """
    event: Event
    r"""The [Event](../guides/core-concepts.mdx#event) resource represents a usage event that has been created for a
    customer. Events are the core of Orb's usage-based billing model, and are used to calculate the usage charges for
    a given billing period.
    """
    invoice: Invoice
    r"""An [`Invoice`](../guides/concepts#invoice) is a fundamental billing entity, representing the request for payment for
    a single subscription. This includes a set of line items, which correspond to prices in the subscription's plan and
    can represent fixed recurring fees or usage-based fees. They are generated at the end of a billing period, or as
    the result of an action, such as a cancellation.
    """
    item: Item
    r"""The Item resource represents a sellable product or good. Items are associated with all line items, billable metrics,
    and prices and are used for defining external sync behavior for invoices and tax calculation purposes.
    """
    metric: Metric
    r"""The Metric resource represents a calculation of a quantity based on events.
    Metrics are defined by the query that transforms raw usage events into meaningful values for your customers.
    """
    plan: Plan
    r"""The [Plan](../guides/core-concepts.mdx#plan-and-price) resource represents a plan that can be subscribed to by a
    customer. Plans define the billing behavior of the subscription. You can see more about how to configure prices
    in the [Price resource](/reference/price).
    """
    price: Price
    r"""The Price resource represents a price that can be billed on a subscription, resulting in a charge on an invoice in
    the form of an invoice line item. Prices take a quantity and determine an amount to bill.

    Orb supports a few different pricing models out of the box. Each of these models is serialized differently in a
    given Price object. The model_type field determines the key for the configuration object that is present.

    ## Unit pricing
    With unit pricing, each unit costs a fixed amount.
    ```json
    {
        ...
        \"model_type\": \"unit\",
        \"unit_config\": {
            \"unit_amount\": \"0.50\"
        }
        ...
    }
    ```

    ## Tiered pricing
    In tiered pricing, the cost of a given unit depends on the tier range that it falls into, where each tier range is
    defined by an upper and lower bound. For example, the first ten units may cost $0.50 each and all units thereafter
    may cost $0.10 each.
    ```json
    {
        ...
        \"model_type\": \"tiered\",
        \"tiered_config\": {
            \"tiers\": [
                {
                    \"first_unit\": 1,
                    \"last_unit\": 10,
                    \"unit_amount\": \"0.50\"
                },
                {
                    \"first_unit\": 11,
                    \"last_unit\": null,
                    \"unit_amount\": \"0.10\"
                }
            ]
        }
        ...
    ```

    ## Bulk pricing
    Bulk pricing applies when the number of units determine the cost of all units. For example, if you've bought less
    than 10 units, they may each be $0.50 for a total of $5.00. Once you've bought more than 10 units, all units may
    now be priced at $0.40 (i.e. 101 units total would be $40.40).
    ```json
    {
        ...
        \"model_type\": \"bulk\",
        \"bulk_config\": {
            \"tiers\": [
                {
                    \"maximum_units\": 10,
                    \"unit_amount\": \"0.50\"
                },
                {
                    \"maximum_units\": 1000,
                    \"unit_amount\": \"0.40\"
                }
            ]
        }
        ...
    }
    ```

    ## Package pricing
    Package pricing defines the size or granularity of a unit for billing purposes. For example, if the package size is
    set to 5, then 4 units will be billed as 5 and 6 units will be billed at 10.
    ```json
    {
        ...
        \"model_type\": \"package\",
        \"package_config\": {
            \"package_amount\": \"0.80\",
            \"package_size\": 10
        }
        ...
    }
    ```

    ## BPS pricing
    BPS pricing specifies a per-event (e.g. per-payment) rate in one hundredth of a percent (the number of basis points
    to charge), as well as a cap per event to assess. For example, this would allow you to assess a fee of 0.25% on
    every payment you process, with a maximum charge of $25 per payment.
    ```json
    {
        ...
        \"model_type\": \"bps\",
        \"bps_config\": {
           \"bps\": 125,
           \"per_unit_maximum\": \"11.00\"
        }
        ...
     }
    ```

    ## Bulk BPS pricing
    Bulk BPS pricing specifies BPS parameters in a tiered manner, dependent on the total quantity across all events.
    Similar to bulk pricing, the BPS parameters of a given event depends on the tier range that the billing period
    falls into. Each tier range is defined by an upper bound. For example, after $1.5M of payment volume is reached,
    each individual payment may have a lower cap or a smaller take-rate.
    ```json
        ...
        \"model_type\": \"bulk_bps\",
        \"bulk_bps_config\": {
            \"tiers\": [
               {
                    \"maximum_amount\": \"1000000.00\",
                    \"bps\": 125,
                    \"per_unit_maximum\": \"19.00\"
               },
              {
                    \"maximum_amount\": null,
                    \"bps\": 115,
                    \"per_unit_maximum\": \"4.00\"
                }
            ]
        }
        ...
    }
    ```

    ## Tiered BPS pricing
    Tiered BPS pricing specifies BPS parameters in a graduated manner, where an event's applicable parameter is a
    function of its marginal addition to the period total. Similar to tiered pricing, the BPS parameters of a given
    event depends on the tier range that it falls into, where each tier range is defined by an upper and lower bound.
    For example, the first few payments may have a 0.8 BPS take-rate and all payments after a specific volume may
    incur a take-rate of 0.5 BPS each.
    ```json
        ...
        \"model_type\": \"tiered_bps\",
        \"tiered_bps_config\": {
            \"tiers\": [
               {
                    \"minimum_amount\": \"0\",
                    \"maximum_amount\": \"1000000.00\",
                    \"bps\": 125,
                    \"per_unit_maximum\": \"19.00\"
               },
              {
                    \"minimum_amount\": \"1000000.00\",
                    \"maximum_amount\": null,
                    \"bps\": 115,
                    \"per_unit_maximum\": \"4.00\"
                }
            ]
        }
        ...
    }
    ```

    ## Matrix pricing
    Matrix pricing defines a set of unit prices in a one or two-dimensional matrix. `dimensions` defines the two event
    property values evaluated in this pricing model. In a one-dimensional matrix, the second value is `null`. Every
    configuration has a list of `matrix_values` which give the unit prices for specified property values. In a
    one-dimensional matrix, the matrix values will have `dimension_values` where the second value of the pair is null.
    If an event does not match any of the dimension values in the matrix, it will resort to the `default_unit_amount`.
    ```json
    {
        \"model_type\": \"matrix\"
        \"matrix_config\": {
            \"default_unit_amount\": \"3.00\",
            \"dimensions\": [
                \"cluster_name\",
                \"region\"
            ],
            \"matrix_values\": [
                {
                    \"dimension_values\": [
                        \"alpha\",
                        \"west\"
                    ],
                    \"unit_amount\": \"2.00\"
                },
                ...
            ]
        }
    }
    ```

    ### Fixed fees
    Fixed fees are prices that are applied independent of usage quantities, and follow unit pricing. They also have an
    additional parameter `fixed_price_quantity`. If the Price represents a fixed cost, this represents the quantity of
    units applied.
    ```json
    {
        ...
        \"id\": \"price_id\",
        \"model_type\": \"unit\",
        \"unit_config\": {
           \"unit_amount\": \"2.00\"
        },
        \"fixed_price_quantity\": 3.0
        ...
    }
    ```
    """
    price_interval: PriceInterval
    r"""The Price Interval resource represents a period of time for which a price will bill on a subscription. A
    subscription’s price intervals define its billing behavior.
    """
    subscription: Subscription
    r"""A [subscription](../guides/core-concepts.mdx#subscription) represents the purchase of a plan by a customer.

    By default, subscriptions begin on the day that they're created and renew automatically for each billing cycle at
    the cadence that's configured in the plan definition.

    Subscriptions also default to **beginning of month alignment**, which means the first invoice issued for the
    subscription will have pro-rated charges between the `start_date` and the first of the following month. Subsequent
    billing periods will always start and end on a month boundary (e.g. subsequent month starts for monthly billing).

    Depending on the plan configuration, any _flat_ recurring fees will be billed either at the beginning (in-advance)
    or end (in-arrears) of each billing cycle. Plans default to **in-advance billing**. Usage-based fees are billed in
    arrears as usage is accumulated. In the normal course of events, you can expect an invoice to contain usage-based
    charges for the previous period, and a recurring fee for the following period.
    """

    sdk_configuration: SDKConfiguration

    def __init__(self,
                 api_key_auth: str,
                 server_idx: int = None,
                 server_url: str = None,
                 url_params: dict[str, str] = None,
                 client: requests_http.Session = None,
                 retry_config: utils.RetryConfig = None
                 ) -> None:
        """Instantiates the SDK configuring it with the provided parameters.
        
        :param api_key_auth: The api_key_auth required for authentication
        :type api_key_auth: str
        :param server_idx: The index of the server to use for all operations
        :type server_idx: int
        :param server_url: The server URL to use for all operations
        :type server_url: str
        :param url_params: Parameters to optionally template the server URL with
        :type url_params: dict[str, str]
        :param client: The requests.Session HTTP client to use for all operations
        :type client: requests_http.Session
        :param retry_config: The utils.RetryConfig to use globally
        :type retry_config: utils.RetryConfig
        """
        if client is None:
            client = requests_http.Session()
        
        
        security_client = utils.configure_security_client(client, shared.Security(api_key_auth = api_key_auth))
        
        
        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        self.sdk_configuration = SDKConfiguration(client, security_client, server_url, server_idx, retry_config=retry_config)
       
        self._init_sdks()
    
    def _init_sdks(self):
        self.availability = Availability(self.sdk_configuration)
        self.coupon = Coupon(self.sdk_configuration)
        self.credit = Credit(self.sdk_configuration)
        self.credit_note = CreditNote(self.sdk_configuration)
        self.customer = Customer(self.sdk_configuration)
        self.event = Event(self.sdk_configuration)
        self.invoice = Invoice(self.sdk_configuration)
        self.item = Item(self.sdk_configuration)
        self.metric = Metric(self.sdk_configuration)
        self.plan = Plan(self.sdk_configuration)
        self.price = Price(self.sdk_configuration)
        self.price_interval = PriceInterval(self.sdk_configuration)
        self.subscription = Subscription(self.sdk_configuration)
    


"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from ..shared import amendmentledgerentry as shared_amendmentledgerentry
from ..shared import creditblockexpiryledgerentry as shared_creditblockexpiryledgerentry
from ..shared import decrementledgerentry as shared_decrementledgerentry
from ..shared import expirationchangeledgerentry as shared_expirationchangeledgerentry
from ..shared import incrementledgerentry as shared_incrementledgerentry
from ..shared import paginationmetadata as shared_paginationmetadata
from ..shared import voidinitiatedledgerentry as shared_voidinitiatedledgerentry
from ..shared import voidledgerentry as shared_voidledgerentry
from dataclasses_json import Undefined, dataclass_json
from orb import utils
from typing import Union


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class CreditLedgerEntries:
    data: list[Union[shared_incrementledgerentry.IncrementLedgerEntry, shared_decrementledgerentry.DecrementLedgerEntry, shared_expirationchangeledgerentry.ExpirationChangeLedgerEntry, shared_creditblockexpiryledgerentry.CreditBlockExpiryLedgerEntry, shared_voidledgerentry.VoidLedgerEntry, shared_voidinitiatedledgerentry.VoidInitiatedLedgerEntry, shared_amendmentledgerentry.AmendmentLedgerEntry]] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('data') }})
    pagination_metadata: shared_paginationmetadata.PaginationMetadata = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('pagination_metadata') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from .add_edit_price_intervals import *
from .amend_event import *
from .amend_usage import *
from .amend_usage_external_customer_id import *
from .archive_coupon import *
from .cancel_subscription import *
from .close_backfill import *
from .create_backfill import *
from .create_coupon import *
from .create_customer import *
from .create_customer_balance_transaction import *
from .create_invoice import *
from .create_invoice_line_item import *
from .create_ledger_entry import *
from .create_ledger_entry_external_id import *
from .create_metric import *
from .create_plan import *
from .create_price import *
from .create_subscription import *
from .delete_customer import *
from .deprecate_event import *
from .fetch_backfill import *
from .fetch_coupon import *
from .fetch_credit_note import *
from .fetch_customer import *
from .fetch_customer_costs import *
from .fetch_customer_costs_external_id import *
from .fetch_customer_credits import *
from .fetch_customer_credits_external_id import *
from .fetch_customer_credits_ledger import *
from .fetch_customer_credits_ledger_external_id import *
from .fetch_customer_external_id import *
from .fetch_invoice import *
from .fetch_item import *
from .fetch_metric import *
from .fetch_plan import *
from .fetch_plan_external_id import *
from .fetch_price import *
from .fetch_price_external_id import *
from .fetch_subscription import *
from .fetch_subscription_costs import *
from .fetch_subscription_schedule import *
from .fetch_subscription_usage import *
from .fetch_upcoming_invoice import *
from .ingest import *
from .issue_invoice import *
from .list_backfills import *
from .list_balance_transactions import *
from .list_coupon_subscriptions import *
from .list_coupons import *
from .list_credit_notes import *
from .list_customers import *
from .list_invoices import *
from .list_items import *
from .list_metrics import *
from .list_plans import *
from .list_prices import *
from .list_subscriptions import *
from .mark_invoice_as_paid import *
from .ping import *
from .revert_backfill import *
from .schedule_plan_change import *
from .search_events import *
from .trigger_phase import *
from .unschedule_cancellation import *
from .unschedule_fixed_fee_quantity import *
from .unschedule_plan_change import *
from .update_customer import *
from .update_customer_external_id import *
from .update_fixed_fee_quantity import *
from .update_plan import *
from .update_plan_external import *
from .void_invoice import *

__all__ = ["AddEditPriceIntervalsRequest","AddEditPriceIntervalsResponse","AmendEventRequest","AmendEventResponse","AmendUsageExternalCustomerIDRequest","AmendUsageExternalCustomerIDResponse","AmendUsageRequest","AmendUsageResponse","ArchiveCouponRequest","ArchiveCouponResponse","CancelSubscriptionRequest","CancelSubscriptionResponse","CloseBackfillRequest","CloseBackfillResponse","CreateBackfillResponse","CreateCouponResponse","CreateCustomerBalanceTransactionRequest","CreateCustomerBalanceTransactionResponse","CreateCustomerResponse","CreateInvoiceLineItemResponse","CreateInvoiceResponse","CreateLedgerEntryExternalIDRequest","CreateLedgerEntryExternalIDResponse","CreateLedgerEntryRequest","CreateLedgerEntryResponse","CreateMetricResponse","CreatePlanResponse","CreatePriceResponse","CreateSubscriptionResponse","DeleteCustomerRequest","DeleteCustomerResponse","DeprecateEventRequest","DeprecateEventResponse","FetchBackfillRequest","FetchBackfillResponse","FetchCouponRequest","FetchCouponResponse","FetchCreditNoteRequest","FetchCreditNoteResponse","FetchCustomerCostsExternalIDRequest","FetchCustomerCostsExternalIDResponse","FetchCustomerCostsExternalIDViewModeViewMode","FetchCustomerCostsRequest","FetchCustomerCostsResponse","FetchCustomerCostsViewModeViewMode","FetchCustomerCreditsExternalIDRequest","FetchCustomerCreditsExternalIDResponse","FetchCustomerCreditsLedgerEntryStatusEntryStatus","FetchCustomerCreditsLedgerEntryTypeEntryType","FetchCustomerCreditsLedgerExternalIDEntryStatusEntryStatus","FetchCustomerCreditsLedgerExternalIDEntryTypeEntryType","FetchCustomerCreditsLedgerExternalIDRequest","FetchCustomerCreditsLedgerExternalIDResponse","FetchCustomerCreditsLedgerRequest","FetchCustomerCreditsLedgerResponse","FetchCustomerCreditsRequest","FetchCustomerCreditsResponse","FetchCustomerExternalIDRequest","FetchCustomerExternalIDResponse","FetchCustomerRequest","FetchCustomerResponse","FetchInvoiceRequest","FetchInvoiceResponse","FetchItemRequest","FetchItemResponse","FetchMetricRequest","FetchMetricResponse","FetchPlanExternalIDRequest","FetchPlanExternalIDResponse","FetchPlanRequest","FetchPlanResponse","FetchPriceExternalIDRequest","FetchPriceExternalIDResponse","FetchPriceRequest","FetchPriceResponse","FetchSubscriptionCostsRequest","FetchSubscriptionCostsResponse","FetchSubscriptionCostsViewModeViewMode","FetchSubscriptionRequest","FetchSubscriptionResponse","FetchSubscriptionScheduleRequest","FetchSubscriptionScheduleResponse","FetchSubscriptionUsageGranularityGranularity","FetchSubscriptionUsageRequest","FetchSubscriptionUsageResponse","FetchSubscriptionUsageViewModeViewMode","FetchUpcomingInvoiceRequest","FetchUpcomingInvoiceResponse","IngestRequest","IngestResponse","IssueInvoiceRequest","IssueInvoiceResponse","ListBackfillsRequest","ListBackfillsResponse","ListBalanceTransactionsRequest","ListBalanceTransactionsResponse","ListCouponSubscriptionsRequest","ListCouponSubscriptionsResponse","ListCouponsRequest","ListCouponsResponse","ListCreditNotesRequest","ListCreditNotesResponse","ListCustomersRequest","ListCustomersResponse","ListInvoicesDateTypeDateType","ListInvoicesRequest","ListInvoicesResponse","ListInvoicesStatus","ListInvoicesStatusStatus","ListItemsRequest","ListItemsResponse","ListMetricsRequest","ListMetricsResponse","ListPlansRequest","ListPlansResponse","ListPlansStatusStatus","ListPricesRequest","ListPricesResponse","ListSubscriptionsRequest","ListSubscriptionsResponse","ListSubscriptionsStatusStatus","MarkInvoiceAsPaidRequest","MarkInvoiceAsPaidResponse","PingResponse","RevertBackfillRequest","RevertBackfillResponse","SchedulePlanChangeRequest","SchedulePlanChangeResponse","SearchEventsRequest","SearchEventsResponse","TriggerPhaseRequest","TriggerPhaseResponse","UnscheduleCancellationRequest","UnscheduleCancellationResponse","UnscheduleFixedFeeQuantityRequest","UnscheduleFixedFeeQuantityResponse","UnschedulePlanChangeRequest","UnschedulePlanChangeResponse","UpdateCustomerExternalIDRequest","UpdateCustomerExternalIDResponse","UpdateCustomerRequest","UpdateCustomerResponse","UpdateFixedFeeQuantityRequest","UpdateFixedFeeQuantityResponse","UpdatePlanExternalRequest","UpdatePlanExternalResponse","UpdatePlanRequest","UpdatePlanResponse","VoidInvoiceRequest","VoidInvoiceResponse"]



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from orb import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class PlanMinified:
    external_plan_id: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('external_plan_id') }})
    r"""An optional user-defined ID for this plan resource, used throughout the system as an alias for this Plan. Use this field to identify a plan by an existing identifier in your system."""
    id: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id') }})
    name: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from orb import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class PaginationMetadata:
    has_more: bool = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('has_more') }})
    next_cursor: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('next_cursor') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from orb import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class PriceGroup:
    grouping_key: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('grouping_key') }})
    r"""Grouping key to break down a single price's costs"""
    grouping_value: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('grouping_value') }})
    secondary_grouping_key: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('secondary_grouping_key') }})
    r"""If the price is a matrix price, this is the second dimension key"""
    secondary_grouping_value: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('secondary_grouping_value') }})
    total: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('total') }})
    r"""Total costs for this group for the timeframe. Note that this does not account for any minimums or discounts."""
    

